@@ -1,4 +1,5 @@
 from twisted.python import log
+from twisted.internet import defer
 from interface import IRouterContainer
 
 from txtorcon.util import find_keywords
@@ -58,10 +59,11 @@ class Circuit(object):
     def __init__(self, routercontainer):
         """
         :param routercontainer: should implement
-        :class:`txtorcon.interface.IRouterContainer`
+        :class:`txtorcon.interface.IRouterContainer`.
         """
         self.listeners = []
         self.router_container = IRouterContainer(routercontainer)
+        self.torstate = routercontainer
         self.path = []
         self.streams = []
         self.purpose = None
@@ -69,6 +71,10 @@ class Circuit(object):
         self.state = 'UNKNOWN'
         self.build_flags = []
 
+        ## this is used to hold a Deferred that will callback() when
+        ## this circuit is being CLOSED or FAILED.
+        self._closing_deferred = None
+
     def listen(self, listener):
         if listener not in self.listeners:
             self.listeners.append(listener)
@@ -76,6 +82,30 @@ class Circuit(object):
     def unlisten(self, listener):
         self.listeners.remove(listener)
 
+    def close(self, **kw):
+        """
+        This asks Tor to close the underlying circuit object. See
+        :method:`txtorcon.torstate.TorState.close_circuit`
+        for details.
+
+        You may pass keyword arguments to take care of any Flags Tor
+        accepts for the CLOSECIRCUIT command. Currently, this is only
+        "IfUnused". So for example: circ.close(IfUnused=True)
+
+        :return: Deferred which callbacks with this Circuit instance
+        ONLY after Tor has confirmed it is gone (not simply that the
+        CLOSECIRCUIT command has been queued). This could be a while
+        if you included IfUnused.
+        """
+
+        self._closing_deferred = defer.Deferred()
+
+        def close_command_is_queued(*args):
+            return self._closing_deferred
+        d = self.torstate.close_circuit(self, **kw)
+        d.addCallback(close_command_is_queued)
+        return self._closing_deferred
+
     def _create_flags(self, kw):
         "this clones the kw dict, adding a lower-case version of every key (duplicated in stream.py; put in util?)"
 
@@ -117,6 +147,7 @@ class Circuit(object):
                 log.err(RuntimeError("Circuit is %s but still has %d streams" %
                                      (self.state, len(self.streams))))
             flags = self._create_flags(kw)
+            self.maybe_call_closing_deferred()
             [x.circuit_closed(self, **flags) for x in self.listeners]
 
         elif self.state == 'FAILED':
@@ -124,8 +155,19 @@ class Circuit(object):
                 log.err(RuntimeError("Circuit is %s but still has %d streams" %
                                      (self.state, len(self.streams))))
             flags = self._create_flags(kw)
+            self.maybe_call_closing_deferred()
             [x.circuit_failed(self, **flags) for x in self.listeners]
 
+    def maybe_call_closing_deferred(self):
+        """
+        Used internally to callback on the _closing_deferred if it
+        exists.
+        """
+
+        if self._closing_deferred:
+            self._closing_deferred.callback(self)
+            self._closing_deferred = None
+
     def update_path(self, path):
         """
         There are EXTENDED messages which don't include any routers at
@@ -143,8 +185,7 @@ class Circuit(object):
 
         oldpath = self.path
         self.path = []
-        for router in path:
-            p = router[:41]
+        for p in path:
             if p[0] != '$':
                 break
 
