@@ -131,6 +131,22 @@ class ICircuitContainer(Interface):
     def find_circuit(id):
         ":return: a circuit for the id, or exception."
 
+    def close_circuit(circuit, **kwargs):
+        """
+        Close a circuit.
+        :return: a Deferred which callbacks when the closing process
+        is started (not necessarily finished inside Tor).
+        """
+
+    ## FIXME do we need an IStreamContainer that Stream instances get?
+    ## (Currently, they get an ICircuitContainer...)
+    def close_stream(stream, **kwargs):
+        """
+        Close a stream.
+        :return: a Deferred which callbacks when the closing process
+        is started (not necessarily finished inside Tor).
+        """
+
 
 class ICircuitListener(Interface):
     """
@@ -261,6 +277,18 @@ class ITorControlProtocol(Interface):
         example.
         """
 
+    def close_circuit(circuit):
+        """
+        Asks Tor to close the circuit. Note that the Circuit instance
+        is only removed as a result of the next CIRC CLOSED event. The
+        Deferred returned from this method callbacks when the
+        CLOSECIRCUIT command has successfully executed, not when the
+        circuit is actually gone.
+
+        If you wish to know when this circuit is actually gone, add an
+        ICircuitListener and wait for circuit_closed()
+        """
+
     def add_circuit_listener(icircuitlistener):
         """
         Add an implementor of :class:`txtorcon.interface.ICircuitListener`
