@@ -36,6 +36,11 @@ def DefaultTCP4EndpointGenerator(*args, **kw):
     Default generator used to create server-side TCP4ServerEndpoint
     instances. Sets interface='127.0.0.1' instead of the default ('',
     which means all).
+
+    (We use a generator so we can try random ports in case something
+    is listening on one; perhaps it would be more straightfoward to
+    simply allow an endpoint to be passed to TCPHiddenServiceEndpoint
+    instead and fail right away in case something is listening?)
     """
 
     kw['interface'] = '127.0.0.1'
@@ -211,17 +216,34 @@ class TCPHiddenServiceEndpoint(object):
         d.addCallback(self._create_listener).addErrback(self._retry_local_port)
         return d
 
+    def check_local_endpoint(self, ep):
+        """
+        This tries to sanity-check ep to see if it's a local address
+        or not and returns True or False.
+
+        Currently, 'local' means either it starts with '127.' or is
+        '::1'
+        """
+
+        if hasattr(ep, '_interface'):
+            is_local = ep._interface.startswith('127.') or ep._interface == '::1'
+            if not is_local:
+                return False
+        return True
+
     def _create_listener(self, proto):
         """
         Creates the local TCP4ServerEndpoint instance, returning a
         Deferred delivering an :api:`twisted.internet.interfaces.IPort` instance that also has
-        :meth:`TCP4HiddenServiceEndpoint._add_attributes` called
+        :meth:`TCPHiddenServiceEndpoint._add_attributes` called
         against it (adds `onion_uri` and `onion_private_key` members).
         """
 
         self._update_onion(self.hiddenservice.dir)
 
         self.tcp_endpoint = self.endpoint_generator(self.reactor, self.listen_port)
+        if not self.check_local_endpoint(self.tcp_endpoint):
+            raise RuntimeError("Endpoint doesn't appear to be a local interface.")
         d = self.tcp_endpoint.listen(self.protocolfactory)
         d.addCallback(self._add_attributes).addErrback(self._retry_local_port)
         return d
@@ -293,6 +315,7 @@ class TorProcessProtocol(protocol.ProcessProtocol):
         self.stdout = []
 
         self._setup_complete = False
+        self._did_timeout = False
         self._timeout_delayed_call = None
         if timeout:
             if not ireactortime:
@@ -326,8 +349,11 @@ class TorProcessProtocol(protocol.ProcessProtocol):
         A timeout was supplied during setup, and the time has run out.
         """
 
-        self.connected_cb.errback(RuntimeError("Timed out waiting for Tor to launch."))
-        self.transport.loseConnection()
+        try:
+            self.transport.signalProcess('KILL')
+        except error.ProcessExitedAlready:
+            self.transport.loseConnection()
+        self._did_timeout = True
 
     def errReceived(self, data):
         """
@@ -356,7 +382,14 @@ class TorProcessProtocol(protocol.ProcessProtocol):
         if isinstance(status.value, error.ProcessDone):
             return
 
-        raise RuntimeError('\n'.join(self.stdout) + "\n\nTor exited with error-code %d" % status.value.exitCode)
+        if status.value.exitCode is None:
+            if self._did_timeout:
+                err = RuntimeError('\n'.join(self.stdout) + "\n\nTimeout waiting for Tor launch..")
+            else:
+                err = RuntimeError('\n'.join(self.stdout) + "\n\nTor was killed (%s)." % status.value.signal)
+        else:
+            err = RuntimeError('\n'.join(self.stdout) + "\n\nTor exited with error-code %d" % status.value.exitCode)
+        self.connected_cb.errback(err)
 
     def progress(self, percent, tag, summary):
         """
@@ -448,7 +481,7 @@ def launch_tor(config, reactor,
 
     :return: a Deferred which callbacks with a TorProcessProtocol
         connected to the fully-bootstrapped Tor; this has a
-        :class:`txtorcon.TorControlProtocol` instance as .protocol. In Tor,
+        :class:`txtorcon.TorControlProtocol` instance as `.tor_protocol`. In Tor,
         ``__OwningControllerProcess`` will be set and TAKEOWNERSHIP will have
         been called, so if you close the TorControlProtocol the Tor should
         exit also (see `control-spec <https://gitweb.torproject.org/torspec.git/blob/HEAD:/control-spec.txt>`_ 3.23).
@@ -779,7 +812,7 @@ class HiddenService(object):
 
     def config_attributes(self):
         """
-        Helper method used by y TorConfig when generating a torrc file.
+        Helper method used by TorConfig when generating a torrc file.
         """
 
         rtn = [('HiddenServiceDir', self.dir)]
@@ -800,22 +833,27 @@ class TorConfig(object):
     call.
 
     Also, it gives easy access to all the configuration options
-    present. This is done with lazy caching: the first time you access
-    a value, it asks the underlying Tor (via TorControlProtocol) and
-    thereafter caches the value; if you change it, a SETCONF is
-    issued.
+    present. This is loaded at "bootstrap" time (when all values are
+    loaded) providing attribute-based access thereafter. If you set an
+    item AND we're bootstrapped to a Tor, THEN that valud is NOT
+    reflected in Tor until you do save() -- and neither is it
+    reflected in the TorConfig instance until that time. So, you might
+    get slightly confusing behavior like: ``config.SOCKSPort=1234;
+    print config.SOCKSPort`` which will still print 9050 or whatever
+    the original value is. (TODO is this really a good idea?
+    Especially since we "need" the other behavior for "build config
+    from scratch" use-case)
 
-    When setting configuration values, they are cached locally and DO
-    NOT AFFECT the running Tor until you call save(). When getting
-    config items they will reflect the current state of Tor
-    (i.e. *not* what's been set since the last save())
+    You may also use this class to construct a configuration from
+    scratch (e.g. to give to :func:`txtorcon.launch_tor`). In this
+    case, values are reflected right away. (If we're not bootstrapped
+    to a Tor, this is the mode).
 
     Note that you do not need to call save() if you're just using
     TorConfig to create a .torrc file or for input to launch_tor().
 
-    FIXME: It also listens on the CONF_CHANGED event to update the
-    cached data in the event other controllers (etc) changed it. (Only
-    exists in Git versions?)
+    This listens for CONF_CHANGED events to update the cached data in
+    the event other controllers (etc) changed it.
 
     FIXME: when is CONF_CHANGED introduced in Tor? Can we do anything
     like it for prior versions?
@@ -907,7 +945,10 @@ class TorConfig(object):
             except KeyError, e:
                 raise AttributeError(str(e))
         else:
-            return self.config[self._find_real_name(name)]
+            rn = self._find_real_name(name)
+            if '_slutty_' in self.__dict__ and rn in self.unsaved:
+                return self.unsaved[rn]
+            return self.config[rn]
 
     def get_type(self, name):
         """
