@@ -4,7 +4,7 @@ from twisted.python import log
 from twisted.internet import defer
 from twisted.internet.interfaces import IProtocolFactory
 from twisted.protocols.basic import LineOnlyReceiver
-from zope.interface import implements
+from zope.interface import implements, implementedBy
 
 from txtorcon.util import hmac_sha256, compare_via_hash
 from txtorcon.log import txtorlog
@@ -139,7 +139,7 @@ def parse_keywords(lines, multiline_values=True):
         if line.strip() == 'OK':
             continue
 
-        if '=' in line:
+        if '=' in line and ' ' not in line.split('=', 1)[0]:
             if key:
                 if key in rtn:
                     if isinstance(rtn[key], types.ListType):
@@ -269,6 +269,9 @@ class TorControlProtocol(LineOnlyReceiver):
                                        self._is_continuation_line,
                                        self._start_command))
 
+        recv.add_transition(Transition(recvmulti,
+                                       self._is_multi_line,
+                                       self._accumulate_response))
         recv.add_transition(Transition(recv,
                                        self._is_continuation_line,
                                        self._accumulate_response))
@@ -463,12 +466,23 @@ class TorControlProtocol(LineOnlyReceiver):
         return self.queue_command('AUTHENTICATE ' + passphrase.encode("hex"))
 
     def quit(self):
+        """
+        Sends the QUIT command, which asks Tor to hang up on this
+        controller connection.
+
+        If you've taken ownership of the Tor to which you're
+        connected, this should also cause it to exit. Otherwise, it
+        won't.
+        """
         return self.queue_command('QUIT')
 
     def queue_command(self, cmd, arg=None):
         """
         returns a Deferred which will fire with the response data when
         we get it
+
+        Note that basically every request is ultimately funelled
+        through this command.
         """
 
         d = defer.Deferred()
@@ -607,6 +621,8 @@ class TorControlProtocol(LineOnlyReceiver):
         raise RuntimeError("The Tor I connected to doesn't support SAFECOOKIE nor COOKIE authentication and I have no password_function specified.")
 
     def _do_password_authentication(self, passwd):
+        if not passwd:
+            raise RuntimeError("No password available.")
         self.authenticate(passwd).addCallback(self._bootstrap).addErrback(self._auth_failed)
 
     def _set_valid_events(self, events):
