@@ -13,42 +13,43 @@ import subprocess
 import struct
 
 try:
-    import GeoIP
-
-    def create_geoip(fname):
-        try:
-            ## It's more "pythonic" to just wait for the exception,
-            ## but GeoIP prints out "Can't open..." messages for you,
-            ## which isn't desired here
-            if not os.path.isfile(fname):
-                raise IOError("Can't find %s" % fname)
-            return GeoIP.open(fname, GeoIP.GEOIP_STANDARD)
-
-        except GeoIP.error:
-            raise IOError("Can't load %s" % fname)
-
+    import GeoIP as _GeoIP
+    GeoIP = _GeoIP
 except ImportError:
-    import pygeoip
-    create_geoip = pygeoip.GeoIP
+    GeoIP = None
 
 city = None
 country = None
 asn = None
 
-try:
-    city = create_geoip("/usr/share/GeoIP/GeoLiteCity.dat")
-except IOError:
-    city = None
 
-try:
-    asn = create_geoip("/usr/share/GeoIP/GeoIPASNum.dat")
-except IOError:
-    asn = None
+def create_geoip(fname):
+    ## It's more "pythonic" to just wait for the exception,
+    ## but GeoIP prints out "Can't open..." messages for you,
+    ## which isn't desired here
+    if not os.path.isfile(fname):
+        raise IOError("Can't find %s" % fname)
 
-try:
-    country = create_geoip("/usr/share/GeoIP/IP.dat")
-except IOError:
-    country = None
+    if GeoIP is None:
+        return None
+
+    try:
+        return GeoIP.open(fname, GeoIP.GEOIP_STANDARD)
+
+    except GeoIP.error:
+        raise IOError("Can't load %s" % fname)
+
+
+def maybe_create_db(path):
+    try:
+        return create_geoip(path)
+    except IOError:
+        return None
+
+city, asn, country = map(maybe_create_db,
+                         ("/usr/share/GeoIP/GeoLiteCity.dat",
+                          "/usr/share/GeoIP/GeoIPASNum.dat",
+                          "/usr/share/GeoIP/IP.dat"))
 
 try:
     import ipaddr as _ipaddr
@@ -63,20 +64,35 @@ def is_executable(path):
 
 
 def find_tor_binary(globs=('/usr/sbin/', '/usr/bin/',
-                           '/Applications/TorBrowser_*.app/Contents/MacOS/')):
-    """Tries to find the tor executable using the shell first or in in the paths
-       whose glob-patterns is in the given 'globs'-tuple.
+                           '/Applications/TorBrowser_*.app/Contents/MacOS/'),
+                    system_tor=True):
     """
+    Tries to find the tor executable using the shell first or in in the
+    paths whose glob-patterns is in the given 'globs'-tuple.
+
+    :param globs:
+        A tuple of shell-style globs of directories to use to find tor
+        (TODO consider making that globs to actual tor binary?)
+
+    :param system_tor:
+        This controls whether bash is used to seach for 'tor' or
+        not. If False, we skip that check and use only the 'globs'
+        tuple.
+    """
+
     # Try to find the tor executable using the shell
-    try:
-        proc = subprocess.Popen(('type -p tor', ), stdout=subprocess.PIPE,
-                                stderr=subprocess.PIPE, shell=True)
-    except OSError:
-        pass
-    else:
-        stdout, _ = proc.communicate()
-        if proc.poll() == 0 and stdout != '':
-            return stdout.strip()
+    if system_tor:
+        try:
+            proc = subprocess.Popen(('type -p tor', ), executable='/bin/bash',
+                                    stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                                    shell=True)
+        except OSError:
+            pass
+        else:
+            stdout, _ = proc.communicate()
+            if proc.poll() == 0 and stdout != '':
+                return stdout.strip()
+
     # the shell may not provide type and tor is usually not on PATH when using
     # the browser-bundle. Look in specific places
     for pattern in globs:
@@ -84,12 +100,14 @@ def find_tor_binary(globs=('/usr/sbin/', '/usr/bin/',
             torbin = os.path.join(path, 'tor')
             if is_executable(torbin):
                 return torbin
+    return None
 
 
 def maybe_ip_addr(addr):
     """
-    Tries to return an IPAddress, otherwise returns a string. I could
-    explicitly check for .exit or .onion at the end instead.
+    Tries to return an IPAddress, otherwise returns a string.
+
+    TODO consider explicitly checking for .exit or .onion at the end?
     """
 
     if ipaddr is not None:
@@ -105,7 +123,10 @@ def find_keywords(args, key_filter=lambda x: not x.startswith("$")):
     This splits up strings like name=value, foo=bar into a dict. Does NOT deal
     with quotes in value (e.g. key="value with space" will not work
 
-    By default, note that it takes OUT any key which starts with $ (i.e. a single dollar sign) since for many use-cases the way Tor encodes nodes with "$hash=name" looks like a keyword argument (but it isn't). If you don't want this, override the "key_filter" argument to this method.
+    By default, note that it takes OUT any key which starts with $ (i.e. a
+    single dollar sign) since for many use-cases the way Tor encodes nodes
+    with "$hash=name" looks like a keyword argument (but it isn't). If you
+    don't want this, override the "key_filter" argument to this method.
 
     :return:
         a dict of key->value (both strings) of all name=value type
@@ -198,12 +219,21 @@ class NetLocation:
         self.city = None
         self.asn = None
 
+        if self.ip is None or self.ip == 'unknown':
+            return
+
         if city:
-            r = city.record_by_addr(self.ip)
+            try:
+                r = city.record_by_addr(self.ip)
+            except:
+                r = None
             if r is not None:
                 self.countrycode = r['country_code']
                 self.latlng = (r['latitude'], r['longitude'])
-                self.city = (r['city'], r['region'])
+                try:
+                    self.city = (r['city'], r['region_code'])
+                except KeyError:
+                    self.city = (r['city'], r['region_name'])
 
         elif country:
             self.countrycode = country.country_code_by_addr(ipaddr)
@@ -212,4 +242,7 @@ class NetLocation:
             self.countrycode = ''
 
         if asn:
-            self.asn = asn.org_by_addr(self.ip)
+            try:
+                self.asn = asn.org_by_addr(self.ip)
+            except:
+                self.asn = None
